
/**
 * We need to properly check for the syntax in some translations.
 *
 * @constructor Syntax
 */

function Syntax() {}

/**
 * Developer sometime defines same key in two different files. But they use two
 * different sets of variables. It should throw an error when that happens.
 * This method helps checking if there is any error duplicate in the source
 * files by providing newTranslations.
 *
 * @param {Object} newTranslations
 * @param {String} key
 * @param {Array} vars
 * @return {Boolean}
 * @api public
 */

Syntax.prototype.hasErrorDuplicate = function(newTranslations, key, variables) {
  if(typeof newTranslations[key] === 'undefined') {
    return false;
  }
  if(variables.length !== newTranslations[key].variables.length) {
    return true;
  }
  for(var i in variables) {
    if(newTranslations[key].variables[i] !== variables[i]) {
      return true;
    }
  }
  return false;
};

/**
 * Export instance
 */

module.exports = new Syntax;

/**
 * Export constructor
 */

module.exports.Syntax = Syntax;

